"use strict";

if (!window.hasOwnProperty("Picasso")) {
    window.Picasso = {};
}
if (!window.Picasso.hasOwnProperty("Pricing")) {
    window.Picasso.Pricing = {};
}

(function() {

    Picasso.Pricing.init = function() {
        Picasso.Pricing.fetchPlans(function(plans) {
            Picasso.Pricing.render(plans);
        });
    };

    Picasso.Pricing.render = function(plans) {
        const monthlyContainer = $("#monthlyPlans");
        const yearlyContainer  = $("#yearlyPlans");

        monthlyContainer.empty();
        yearlyContainer.empty();

        plans.forEach(function(plan) {
            const monthlyCard = createMonthlyPlanCard(plan);
            monthlyContainer.append(monthlyCard);
        });

        plans.forEach(function(plan) {
            const yearlyCard = createYearlyPlanCard(plan);
            yearlyContainer.append(yearlyCard);
        });
    };

    function createMonthlyPlanCard(plan) {
        let card = $("<div>", { class: "plan-card" });

        let contentDiv = $("<div>");
        card.append(contentDiv);

        contentDiv.append(
            $("<div>", { class: "plan-title" }).text(plan.getName())
        );

        contentDiv.append(
            $("<div>", { class: "plan-price" }).text(`$${plan.getMonthlyPrice()}`)
        );

        contentDiv.append(
            $("<div>", { class: "plan-duration" }).text(Picasso.Lang.get("label_monthly"))
        );

        contentDiv.append(
            $("<div>", { class: "plan-desc" }).text(plan.getMonthlyDesc())
        );

        let featuresDiv = $("<div>", { class: "plan-features" });
        let ul = $("<ul>");
        Object.entries(plan.getMonthlyMetaData()).forEach(([key, value]) => {
            let displayText = "";
            if (key === "storage") {
                let formattedValue = formatStorage(value);
                displayText = Picasso.Lang.get("label_pricing_storage", formattedValue);
            } else {
                displayText = Picasso.Lang.get("label_" + key, value);
            }
            ul.append($("<li>").text(displayText));
        });

        featuresDiv.append(ul);
        contentDiv.append(featuresDiv);

        let ctaButton = $("<a>", { href: "#", class: "plan-cta" })
            .text(Picasso.Lang.get("label_choose",plan.getName()))
            .data("priceid", plan.getMonthlyPriceId())
            .on("click", function(evt) {
                evt.preventDefault();
                const priceId = $(this).data("priceid");
                Picasso.Pricing.startCheckout(priceId);
            });

        card.append(ctaButton);

        return card;
    }

    function formatStorage(value) {
        let numericValue = (typeof value === "number") ? value : parseFloat(value);

        if (!isNaN(numericValue)) {
            if (numericValue >= 1024) {
                let tb = numericValue / 1024;
                return tb.toFixed(1) + " TB";
            } else {
                return numericValue + " GB";
            }
        }
        return value;
    }


    function createYearlyPlanCard(plan) {
        let card = $("<div>", { class: "plan-card" });

        let contentDiv = $("<div>");
        card.append(contentDiv);


        contentDiv.append(
            $("<div>", { class: "plan-title" }).text(plan.getName())
        );


        contentDiv.append(
            $("<div>", { class: "plan-price" }).text(`${plan.getYearlyPrice()}`)
        );


        contentDiv.append(
            $("<div>", { class: "plan-duration" }).text(Picasso.Lang.get("label_yearly"))
        );


        contentDiv.append(
            $("<div>", { class: "plan-desc" }).text(plan.getYearlyDesc())
        );


        let featuresDiv = $("<div>", { class: "plan-features" });
        let ul = $("<ul>");
        Object.entries(plan.getYearlyMetaData()).forEach(([key, value]) => {
            let displayText = "";
            if (key === "storage") {
                let formattedValue = formatStorage(value);
                displayText = Picasso.Lang.get("label_pricing_storage", formattedValue);
            } else {
                displayText = Picasso.Lang.get("label_" + key, value);
            }
            ul.append($("<li>").text(displayText));
        });

        featuresDiv.append(ul);
        contentDiv.append(featuresDiv);


        let ctaButton = $("<a>", { href: "#", class: "plan-cta" })
            .text(Picasso.Lang.get("label_choose",plan.getName()))
            .data("priceid", plan.getYearlyPriceId())
            .on("click", function(evt) {
                evt.preventDefault();
                const priceId = $(this).data("priceid");
                Picasso.Pricing.startCheckout(priceId);
            });

        card.append(ctaButton);

        return card;
    }

    Picasso.Pricing.startCheckout = function(priceId) {
        Picasso.Helper.fireAjax(
            "/wapi/pricing",
            { action: "createCheckout", priceId: priceId },
            // onSuccess
            function(json, status) {
                if (json && json.sessionUrl) {
                    window.location = json.sessionUrl;
                } else {
                    console.error("No sessionUrl from createCheckout", json);
                }
            },
            // onError
            function(error, httpCode, resp) {
                Picasso.Notification.show(resp.message);
            }
        );
    };

    Picasso.Pricing.fetchPlans = function(onSuccess, onError) {
        Picasso.Helper.fireAjax(
            "/wapi/pricing",
            { action: "getAll" },
            function(json, status) {

                let arr = (json && json.ResultSet && json.ResultSet.Result)
                    ? json.ResultSet.Result
                    : [];

                let plans = arr.map(function(pm) {
                    return new Picasso.Plan(pm);
                });

                if (onSuccess) onSuccess(plans);
            },
            function(error, httpCode, resp) {
                if (onError) onError(error);
            }
        );
    };





    $(document).ready(function(){
        Picasso.Pricing.init();
    });


    document.addEventListener("DOMContentLoaded", function () {
        const urlParams = new URLSearchParams(window.location.search);

        if (urlParams.has('session_id')) {
            Picasso.Notification.show(
                Picasso.Lang.get("notification_success_payment_completed"), "success");
        }

        if (urlParams.has('canceled') && urlParams.get('canceled') === 'true') {
            Picasso.Notification.show(
                Picasso.Lang.get("notification_info_payment_canceled"), "info");
        }
    });

})();
